#
# Fiondella-fishing.R, 19 Jan 25
# Data from
# Software Defect Discovery and Resolution Modeling incorporating Severity
# Maskura Nafreen, Ying Shi and Lance Fiondella

library("colorspace")
library("plyr")


par(bty="l")
par(las=1)
par(pch="+")
pal_col=rainbow(3)


plot_backlog=function(start, end, x_bounds=c(1, 52))
{
# Fit a regression line
   fit_exp=function(df, col_str="black")
   {
   df=subset(df, x>0)
   mod=glm(log(freq) ~ x, data=df, subset=(x <= max(x_bounds)))
   pred=predict(mod, newdata=data.frame(x=x_bounds))
   lines(x_bounds, exp(pred), col=col_str)
   return(NULL)
   }

# Bin reports by week so that the counts are meaningful
weeks=round((end-start+1)/7)
sev_0=count(subset(weeks, fd$Severity == 0))
sev_1=count(subset(weeks, fd$Severity == 1))
sev_2=count(subset(weeks, fd$Severity == 2))

plot(sev_1, log="y", col=pal_col[1],
	xlim=x_bounds,
	xlab="Weeks", ylab="Reports")
fit_exp(sev_1, pal_col[1])

points(sev_0, col=pal_col[2])
fit_exp(sev_0, pal_col[2])

points(sev_2, col=pal_col[3])
fit_exp(sev_2, pal_col[3])

legend("topright", c("Severity 1", "Severity 0", "Severity 2"),
				bty="n", fill=pal_col, cex=1.2)

}



fd=read.csv("Fiondella-CPH.csv")

table(fd$Severity)
#
#   0    1    2 
# 268 1597   59 


# Plot various interval combinations.
# Some domain knowledge would suggest meaningful pairs
plot_backlog(fd$Created.Date, fd$Date.Assigned, c(1, 20))
plot_backlog(fd$Date.Assigned, fd$Date.BuildIntegration, c(1, 80))

dev.copy(dev=png, file="Assigned-BuldInt-time.png")
dev.off()

plot_backlog(fd$Date.Assigned, fd$Date.Canceled, c(1, 80))
plot_backlog(fd$Date.BuildIntegration, fd$Date.InTest, c(1, 80))
plot_backlog(fd$Date.ReadyForTest, fd$Date.TestCompleted, c(1, 80))

